<?php

require_once('connect.client.php');

// check command line parameters are correct:
if (count($argv) != 2) {
    echo "Usage: connect.report.php <employee_number>";
} else if (!is_numeric($argv[1])) {
    echo "Error: Employee Number must be numeric";
} else {
    // log in
    $factory = new TimeTrexCommandLineClientFactory();
    $client = $factory->makeTimeTrexClient();
    $api_session = $client->run();
    if ($client->getTimeTrexSessionId()) {
        $employeeId = (int) $argv[1];
        $user = new TimeTrexClientUser();
        $user_data = $user->getUserData($employeeId);
        if ($user->getResult() == TimeTrexClientUser::SUCCESS) {
            $report = new TimeTrexClientReport();
            $continue = $report->Display($user_data);
        }
    } else {
        echo "Error: could not log in. Please check logindetails.php.";
    }
    $api_session->Logout();
}

class TimeTrexClientReport {

    // display the report for the current Punch ID ( == employee number)    
    // parameter added for quick display (i.e. no long pause to examine the report)
    public function Display($user_data) {
        $success = false;
        $input = $this->getReportData($user_data[0]['employee_number']);
        if ($input != "") {
            $this->processReportData($input, $user_data[0]);
            $success = true;
        }
        return $success;
    }

    // authenticate and then obtain the report data needed to get all the punch info.
    // return empty string if it fails.
    private function getReportData($employeeNumber) {
        $input = "";
        $report_obj = new TimeTrexClientAPI('PunchSummaryReport');
        if ($report_obj == false) {
            echo "CONNECTION ERROR - report_obj";
        } else {
            $config = $report_obj->getTemplate('by_employee+punch_summary+total_time')->getResult();
            if ($config == false) {
                echo "CONNECTION ERROR - getTemplate";
            } else {
                //adjust template
                $config['-1010-time_period']['time_period'] = 'this_pay_period';
                $config['employee_number'] = $employeeNumber;

                //let the server generate the report
                $result = $report_obj->getPunchSummaryReport($config, 'csv');
                if ($result == false) {
                    echo "CONNECTION ERROR - getPunchSummaryReport";
                    $this->result = TimeTrexClientReport::ERROR;
                } else {
                    $result = $result->getResult();
                    if ($result == false) {
                        echo "CONNECTION ERROR - getResult -> getPunchSummaryReport";
                    } else {
                        //parse and display the CSV file
                        //decode report                        
                        $input = base64_decode($result['data']);
                        if ($input == "") {
                            // no data found - add a backslash to indicate still valid:
                            $input = "\\"; // do things better later!                        
                        }
                    }
                }
            }
        }
        return $input;
    }

    // process the raw report data, received and decoded from TimeTrex server.
    // second parameter is the user's details, to be used for the table header.    
    private function processReportData($input, $userDetails) {
        //split the line breaks
        $input = str_replace("\"", "", $input);
        //print table headers
        echo "#{$userDetails['employee_number']}: {$userDetails['last_name']}, {$userDetails['first_name']}\n";
        // print the raw data, which the GUI client must now process: 
        echo $input;
        //done
    }

}

/*********************************************************************************
 * TimeTrex is a Payroll and Time Management program developed by
 * TimeTrex Software Inc. Copyright (C) 2003 - 2013 TimeTrex Software Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY TIMETREX, TIMETREX DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License along
 * with this program; if not, see http://www.gnu.org/licenses or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 *
 * You can contact TimeTrex headquarters at Unit 22 - 2475 Dobbin Rd. Suite
 * #292 Westbank, BC V4T 2E9, Canada or at email address info@timetrex.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by TimeTrex" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by TimeTrex".
 ********************************************************************************/
//
// This TimeTrex client was created by Pro-DVD for their customers. 
// Based upon the work of "EricM" as detailed here: 
// http://forums.timetrex.com/viewtopic.php?p=17581#p17581
//